@@ -269,6 +269,52 @@ sub draw_text_cell {
 
 ################################################################################
 
+=head1 draw_checkbox_cell
+
+   (,  )  checkbox' . 
+
+=head2 
+
+=over
+
+=item name
+
+ CGI-
+
+=item checked
+
+  checkbox  
+
+=item attributes
+
+ HTML-   C<td>.
+
+=back
+
+=head2 
+
+	draw_text_cell ({ 
+		label   => '$1 000 000',
+		href    => '/?type=bank&action=pillage',
+		target  => 'invisible',
+		max_len => 255,
+		attributes => {
+			width => '1%',
+			align => 'right',
+		},
+		a_class => 'red_hot_link'
+	});
+
+=cut
+
+################################################################################
+
+sub draw_checkbox_cell {
+	drawer_call ('draw_checkbox_cell', @_);	
+}
+
+################################################################################
+
 =head1 draw_tr
 
      .
@@ -530,6 +576,10 @@ sub draw_toolbar_button {
 
   CGI-
 
+=item value
+
+  
+
 =back
 
 =head2 
@@ -670,6 +720,10 @@ sub draw_row_buttons {
 
 ,     '',     C<bottom_toolbar>.
 
+=item name
+
+ .   'form'.
+
 =back
 
 =head2 
@@ -739,6 +793,38 @@ sub draw_form_field_string {
 
 ################################################################################
 
+=head1 draw_form_field_button
+
+    'button'.   - C<draw_form>.
+
+=head2 
+
+=over
+
+=item name
+
+  <$data>.
+
+=item value
+
+ HTML- C<value>.    C<name>.
+
+=item onclick
+
+  C<onclick>.
+
+=back
+
+=cut
+
+################################################################################
+
+sub draw_form_field_button {
+	drawer_call ('draw_form_field_button', @_);
+}
+
+################################################################################
+
 =head1 draw_form_field_hgroup
 
     'text':  ,      .   - C<draw_form>.
@@ -783,6 +869,14 @@ sub draw_form_field_hgroup {
 
  
 
+=item rows
+
+ .   25.
+
+=item cols
+
+ .   60.
+
 =back
 
 =cut
@@ -1054,6 +1148,21 @@ sub draw_esc_toolbar {
     ""  "".     
 C<draw_form>    C<esc>.
 
+:
+
+=over
+
+=item esc
+
+   ''.
+
+=item name
+
+ .   'form'.
+
+=back
+
+
 =cut
 
 ################################################################################
